## Script to estimate parameters for the quad-based model using STAN

library(rstan)
library(parallel)
library(ggmcmc)

## Set leave_out_year for validation from command line prompt
args <- commandArgs(trailingOnly = F)
myargument <- args[length(args)]
myargument <- sub("-","",myargument)
leave_out_year_id <- as.numeric(myargument)
leave_out_year_id <- 1

priors_df <- read.csv("all_maxlppds.csv")
priors <- subset(priors_df, vital=="cover")

##  Read in data
#bring in data
allD <- read.csv("quadAllCover.csv")
allD <- allD[,2:ncol(allD)] #get rid of X ID column
sppList <- as.character(unique(allD$Species))

climD <- read.csv("Climate.csv")
clim_vars <- c("pptLag", "ppt1", "ppt2", "TmeanSpr1", "TmeanSpr2")

backD <- data.frame(climYear=NA,
                    quad = NA,
                    year= NA,
                    totCover= NA,
                    Species= NA,
                    propCover= NA,
                    lag.cover= NA,
                    pptLag= NA,
                    ppt1= NA,
                    TmeanSpr1= NA,
                    ppt2= NA,
                    TmeanSpr2= NA,
                    TmeanSum1= NA,
                    TmeanSum2= NA,
                    yearID= NA,
                    group = NA,
                    percCover = NA,
                    percLagCover = NA)

#loop through species and remake data frame
for(spp in 1:length(sppList)){
  doSpp <- sppList[spp]
  sppD <- subset(allD, Species==doSpp)
  
  # create lag cover variable
  tmp=sppD[,c("quad","year","totCover")]
  tmp$year=tmp$year+1
  names(tmp)[3]="lag.cover"
  sppD=merge(sppD,tmp,all.x=T)
  
  # merge in climate data
  sppD$climYear=sppD$year+1900-1  
  sppD=merge(sppD,climD,by.x="climYear",by.y="year")
  
  #Growth observations
  growD <- subset(sppD,lag.cover>0 & totCover>0)
  growD$yearID <- growD$year #for random year offset on intercept
  growD$group <- substring(growD$quad, 1, 1)
  growD$percCover <- growD$totCover/10000
  growD$percLagCover <- growD$lag.cover/10000
  backD <- rbind(backD, growD)
}#end species loop
growD_all <- backD[2:nrow(backD),]

year_ids <- unique(growD_all$year)
yearD_all <- subset(growD_all, year!=year_ids[leave_out_year_id])

growD <- subset(yearD_all, Species=="BOGR")
##  Create and scale interaction covariates
growD$ppt1TmeanSpr1 <- growD$ppt1*growD$TmeanSpr1
growD$ppt2TmeanSpr2 <- growD$ppt2*growD$TmeanSpr2
clim_vars_all <- c(clim_vars, "ppt1TmeanSpr1", "ppt2TmeanSpr2")
clim_covs <- growD[,clim_vars_all]

# Get scalers for climate covariates from fitting data
all_scalers <- readRDS("qbm_all_clim_scalers.RDS")
scalers <- subset(all_scalers, species=="BOGR" & yearout==year_ids[leave_out_year_id])
if(nrow(scalers)!=7){ stop("too many or too few covariates") } 

# Scale the covariates
all_covs <- scalers$covariate
for(do_cov in all_covs){
  clim_covs[,do_cov] <- (clim_covs[,do_cov] - as.numeric(subset(scalers, covariate==do_cov)["means"]))/as.numeric(subset(scalers, covariate==do_cov)["sds"])
}

groups <- as.numeric(as.factor(growD$group))
G <- length(unique(growD$group))
Yrs <- length(unique(growD$year))
yid <- as.numeric(as.factor(growD$year))

datalist <- list(N=nrow(growD), Yrs=Yrs, yid=yid,
                 Covs=ncol(clim_covs), Y=growD$percCover, X=log(growD$percLagCover),
                 C=clim_covs, G=G, gid=groups, sd_clim=0.1)
pars=c("a_mu", "a", "b1_mu",  "b1", "b2",
       "tau", "gint")

mcmc_samples <- stan(file="qbm.stan", data=datalist, pars=pars, chains=0)

##  Loop through species and fit the model
for (do_species in sppList){
  growD <- subset(yearD_all, Species==do_species)
  growD$ppt1TmeanSpr1 <- growD$ppt1*growD$TmeanSpr1
  growD$ppt2TmeanSpr2 <- growD$ppt2*growD$TmeanSpr2
  clim_vars_all <- c(clim_vars, "ppt1TmeanSpr1", "ppt2TmeanSpr2")
  clim_covs <- growD[,clim_vars_all]
  scalers <- subset(all_scalers, species==do_species & yearout==year_ids[leave_out_year_id])
  if(nrow(scalers)!=7){ stop("too many or too few covariates") } 
  
  # Scale the covariates
  all_covs <- scalers$covariate
  for(do_cov in all_covs){
    clim_covs[,do_cov] <- (clim_covs[,do_cov] - as.numeric(subset(scalers, covariate==do_cov)["means"]))/as.numeric(subset(scalers, covariate==do_cov)["sds"])
  }
  groups <- as.numeric(as.factor(growD$group))
  G <- length(unique(growD$group))
  Yrs <- length(unique(growD$year))
  
  ## Set reasonable initial values for three chains
  inits <- list()
  inits[[1]] <- list(a_mu=0, a=rep(0,Yrs), b1_mu=0.01, b1=rep(0.01,Yrs),
                     gint=rep(0,G), w=c(0,0), sig_b1=0.5, sig_a=0.5, tau=0.5,
                     sig_G=0.5, b2=rep(0,ncol(clim_covs)))
  inits[[2]] <- list(a_mu=1, a=rep(1,Yrs), b1_mu=1, b1=rep(1,Yrs),
                     gint=rep(1,G), w=c(0.5,0.5), sig_b1=1, sig_a=1, tau=1,
                     sig_G=1, b2=rep(1,ncol(clim_covs)))
  inits[[3]] <- list(a_mu=0.5, a=rep(0.5,Yrs), b1_mu=0.5, b1=rep(0.5,Yrs),
                     gint=rep(0.5,G), w=c(-0.5,-0.5), sig_b1=0.1, sig_a=0.1, tau=0.1, tauSize=0.1,
                     sig_G=0.1, b2=rep(-1,ncol(clim_covs)))
  
  yid <- as.numeric(as.factor(growD$year))
  
  prior_stddev <- as.numeric(subset(priors, species==do_species)["prior_stdev"])
  
  datalist <- list(N=nrow(growD), Yrs=Yrs, yid=yid,
                   Covs=ncol(clim_covs), Y=growD$percCover, X=log(growD$percLagCover),
                   C=clim_covs, G=G, gid=groups, sd_clim=prior_stddev)
  pars=c("a_mu", "a", "b1_mu",  "b1", "b2",
         "tau", "gint")
  
  rng_seed <- 123
  sflist <-
    mclapply(1:3, mc.cores=3,
             function(i) stan(fit=mcmc_samples, data=datalist, pars=pars,
                              seed=rng_seed, chains=1, chain_id=i, refresh=-1,
                              iter=2000, warmup=1000, init=list(inits[[i]])))
  fit <- sflist2stanfit(sflist)
  r_hats <- summary(fit)$summary[,10] 
  write.csv(r_hats, paste("rhat_leaveout", year_ids[leave_out_year_id], "_", do_species, ".csv", sep=""))
  
  long <- ggs(fit)
  saveRDS(long, paste("popgrowth_stanmcmc_", do_species, "_leaveout", year_ids[leave_out_year_id],".RDS", sep=""))
}
